# Copyright (C) 2009 Daniel Maturana
# This file is part of subspaceextract.
#
# subspaceextract is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# subspaceextract is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with subspaceextract. If not, see <http://www.gnu.org/licenses/>.
#
#

import os
import numpy as np
from PIL import Image

class Dataset(object):
    def __init__(self, data, labels=None, img_shape=None):
        if not isinstance(data, np.ndarray):
            raise ValueError("data should be a numpy ndarray.")
        if labels is not None and not isinstance(labels, np.ndarray):
            raise ValueError("labels should be a numpy ndarray.")
        if labels is not None and (data.shape[0] != labels.shape[0]):
            raise ValueError, "data and labels must have the same number of rows."
        self.data = data
        self.labels = labels
        self._img_shape = img_shape
        self._classes = None if labels is None else np.unique(labels)

    def __getitem__(self,i):
        return (self.data[i,:], self.labels[i])

    def __getslice__(self,start,stop):
        return (self.data[start:stop,:], self.labels[start:stop])

    @property
    def shape(self):
        """Shape of the dataset."""
        return self.data.shape

    @property
    def N(self):
        """The number of observations in the dataset."""
        return self.data.shape[0]

    @property
    def D(self):
        """The number of features (dimensionality) of dataset."""
        return self.data.shape[1]

    @property
    def C(self):
        """Number of unique labels (classes) in the dataset"""
        return len(self._classes)

    @property
    def classes(self):
        """List of unique labels (classes)"""
        return self._classes

    @property
    def img_shape(self):
        """Original shape of images in the dataset."""
        return self._img_shape

    @property
    def mean(self):
        if hasattr(self, '_mean'):
            return self._mean
        self._mean = self.data.mean(0)
        return self._mean

def load(path):
    if os.path.isdir(path):
        return from_directory(path)
    if path.endswith('npz'):
        return from_img_npz(path)
    else:
        raise ValueError("I don't know how to read %s"%path)

def from_npz(npz_file):
    npz = np.load(npz_file)
    data = npz['data']
    # ensure data are doubles in 0..1.0
    if data.dtype == np.uint8:
        data = data.astype(np.float64)/255.
    labels = npz['labels']
    # ensure labels are 0-based
    # if labels.min() == 1:
    #     labels -= 1
    if len(labels) != len(data):
        raise ValueError("Data and labels should have same length.")
    return Dataset(data, labels)

def from_img_npz(npz_file):
    npz = np.load(npz_file)
    data = npz['data']
    # ensure data are doubles in 0..1.0
    if data.dtype == np.uint8:
        data = data.astype(np.float64)/255.
    labels = npz['labels']
    img_shape = npz['img_shape']
    # ensure img_size is sane
    if data.shape[1] != img_shape[0]*img_shape[1]:
        raise ValueError("Image size doesn't is incompatible with data dimensionality")
    # ensure labels are 0-based
    # if labels.min() == 1:
    #   labels -= 1
    if len(labels) != len(data):
        raise ValueError("Data and labels should have same length.")
    return Dataset(data, labels, img_shape)

def from_directory(dirpath):
    data = []
    labels = []
    img_shape = None
    subdirs = [sd for sd in os.listdir(dirpath) if os.path.isdir(os.path.join(dirpath, sd))]
    for subdir in subdirs:
        try:
            label = int(subdir)
        except ValueError:
            print "%s is not a integer, ignoring"%(os.path.join(dirpath, subdir))
        files = os.listdir(os.path.join(dirpath,subdir))
        for f in files:
            fullpath = os.path.join(dirpath, subdir, f)
            try:
                im = Image.open(fullpath)
                arr = np.asarray(im, np.double)/255.
                if img_shape and arr.shape != img_shape:
                    raise Exception("Error reading %s: not the same size as rest."%fullpath)
                img_shape = arr.shape
                data.append(arr.T.flatten())
                labels.append(label)
            except IOError, ex:
                print 'I/O Error reading %s:'%fullpath, ex
    data = np.array(data)
    labels = np.array(labels)
    return Dataset(data, labels, img_shape)
