# Copyright (C) 2009 Daniel Maturana
# This file is part of subspaceextract.
#
# subspaceextract is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# subspaceextract is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with subspaceextract. If not, see <http://www.gnu.org/licenses/>.
#
#


#!/usr/bin/env python

import sys
import dataset
import subspaces

help_message=\
"""
Given a set of images, subspace_extract.py extracts
the basis of the following subspaces for the image space:
- PCA aka Eigenfaces
- PCA+LDA subspace aka Fisherfaces
- Random sampling LDA - randomized version of fisherfaces
- Bagged Nullspace LDA - another randomized version of fisherfaces
Note: this program will only extract the basis vectors of the subspace, e.g.
the 'Eigenfaces'. To use these you will probably need to project your data
onto this subspace. If the data matrix X has one image per row and the basis
matrix W has one basis vector per column the projected data is Y = X*W.

Usage: subspace_extract.py PATH SUBSPACE DIMENSIONALITY OUTPUT
- SUBSPACE is fisherfaces, eigenfaces, randomlda or nullda
- DIMENSIONALITY is desired number of basis vectors for output subspace, one
  of 'eigenfaces', 'fisherfaces', 'randomlda' and 'nulllda'
- PATH is directory containing the images. The convention used is that
  the directory contains one subdirectory per class (individual), and is
  subdirectory has integer number as a name. Eg:
  MyFacesDir/
  |-- 06
  |   |-- dsc_0011.jpg
  |   `-- dsc_0012.jpg
  `-- 08
      |-- dsc_0014.jpg
      `-- dsc_0015.jpg
  This datset contains pictures of three individuals (06, 08) and
  two pictures per individual.
- OUTPUT is the name of the output file. Data is stored as a Matlab4 format
  file with two matrices: 'basis', with columns as basis vectors, and 'mean',
  with the mean of the data.
Examples:
python subspace_extract.py eigenfaces 10 MyFacesFolder myeigenfaces.mat
python subspace_extract.py fisherfaces 9 MyFacesFolder myfisherfaces.mat
"""

def main(args):
    if len(args) != 4:
        self.error_message("ERROR: Insufficient arguments. Use --help for help.")
        sys.exit(1)
    ds = dataset.load(args[0])
    subspace_dict = {'eigenfaces': subspaces.PcaSubspace,
            'fisherfaces': subspaces.FisherFaceSubspace,
            'randomlda': subspaces.RandomLdaSubspace,
            'nulllda': subspaces.BaggedNullLdaSubspace}
    try:
        ss_class = subspace_dict[args[1]]
    except KeyError:
        print "ERROR: Unrecognized subspace. Use --help for help."
        sys.exit(1)
    try:
        out_D = int(args[2])
    except ValueError:
        print "ERROR: Invalid dimensionality. Use --help for help."
        sys.exit(1)
    subsp = ss_class(out_D)
    print 'Extracting %s subspace...' % args[1],
    subsp.extract(ds)
    print 'Done.'
    print 'Saving to %s ...' % args[3],
    subsp.save(args[3], 'mat')
    print 'Done.'

if __name__=="__main__":
    if sys.argv[1] == '--help':
        print help_message
        sys.exit(0)
    main(sys.argv[1:])
